@@ -68,7 +68,7 @@ use Carp;
 use Mojo::URL;
 use Mojo::UserAgent;
 
-our $VERSION = '1.24_04';
+our $VERSION = '1.25_01';
 
 =item run( @macs )
 
@@ -107,7 +107,7 @@ Return the Mojo::UserAgent object used to fetch resources.
 =cut
 
 sub ua {
-	state $ua = Mojo::UserAgent->new;
+	state $ua = Mojo::UserAgent->new->max_redirects(3);
 	$ua;
 	}
 
@@ -197,6 +197,10 @@ To avoid multiple calls on the network, use C<load_cache> to preload
 the entire OUI space into an in-memory cache. This can take a long
 time over a slow network, though; the file is about 60,000 lines.
 
+Also, the IEEE website has been flaky lately, so loading the cache is
+better. This distribution comes with several versions of the complete
+OUI data file.
+
 =cut
 
 sub fetch_oui {
@@ -259,7 +263,6 @@ sub fetch_oui_from_ieee {
 	my $mac = normalize_mac( shift );
 
 	my @urls = __PACKAGE__->_search_url( $mac );
-say "Search URLs are @urls";
 
 	my $html;
 	URL: foreach my $url ( @urls ) {
@@ -280,22 +283,31 @@ say "Search URLs are @urls";
 
 sub _fetch_oui_from_url {
 	my( $class, $url ) = @_;
+	my $tries = 0;
 
 	return unless defined $url;
 
-	my $tx = __PACKAGE__->ua->get( $url );
-	unless( $tx->success ) {
-		carp "Failed fetching [$url]: " . $tx->res->code;
-		return;
-		}
+	TRY: {
+		my $tx = __PACKAGE__->ua->get( $url );
+		unless( $tx->success ) {
+			if( $tries > 3 ) {
+				carp "Failed fetching [$url]: " . $tx->res->code;
+				return;
+				}
 
-	my $html = $tx->res->body;
-	unless( defined $html ) {
-		carp "No content in response for [$url]!";
-		return;
-		}
+			$tries++;
+			sleep 1 * $tries;
+			redo TRY;
+			}
 
-	return $html;
+		my $html = $tx->res->body;
+		unless( defined $html ) {
+			carp "No content in response for [$url]!";
+			return;
+			}
+
+		return $html;
+		}
 	}
 
 =item fetch_oui_from_cache( MAC )
@@ -334,32 +346,11 @@ sub extract_oui_from_html {
 	my $html = shift;
 	my $lookup_mac = normalize_mac( shift );
 
-	$html =~ s/<pre>.*?$lookup_mac/<pre>$lookup_mac/is;
+	my( $record ) = $html =~ m|<pre>(<b>$lookup_mac</b>.*?)</pre>|is;
+	$record =~ s|</?b>||g;
 
-	# sometimes the HTML returns more than one OUI because
-	# IEEE has a problem parsing their own data when they
-	# have private blocks
-	my( $ouis ) = $html =~ m|<pre>(.*?)</pre>|gs;
-	return unless defined $ouis;
-	$ouis =~ s/<\/?b>//gs; # remove bold around the OUI
-
-
-	my @entries = split /\v+\s*\v+/, $ouis;
-	return unless defined $entries[0];
-	return $entries[0] unless defined $entries[1];
-
-	my $result = $entries[0];
-
-	foreach my $entry ( @entries ) {
-		$entry =~ s/^\s+|\s+$//;
-		my $found_mac = normalize_mac( substr $entry, 0, 8 );
-		if( $found_mac eq $lookup_mac ) {
-			$result = $entry;
-			last;
-			}
-		}
-
-	return $result;
+	return unless defined $record;
+	return $record;
 	}
 
 =item parse_oui( STRING )
@@ -383,7 +374,7 @@ With no arguments, it returns an empty anonymous array.
 sub parse_oui {
 	my $oui = shift;
 	return [] unless $oui;
-
+	$oui =~ s|</?b>||g;
 	my @lines = map { s/^\s+//; $_ ? $_ : () } split /$/m, $oui;
 	splice @lines, 1, 1, ();
 
@@ -448,7 +439,11 @@ sub load_cache {
 			do { local( @ARGV, $/ ) = $source; <> }
 			}
 		else {
-			__PACKAGE__->ua->get( oui_url() )->res->body;
+			#say time . " Fetching URL";
+			my $tx = __PACKAGE__->ua->get( oui_url() );
+			#say time . " Fetched URL";
+			#say "size is " . $tx->res->headers->header( 'content-length' );
+			$tx->res->body;
 			}
 		};
 
@@ -550,7 +545,7 @@ brian d foy C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2004-2015, brian d foy <bdfoy@cpan.org>. All rights reserved.
+Copyright © 2004-2015, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
